use crate::json_util::get_json_object;
use crate::json_util::get_json_string;

pub struct PullRequestEvent<'a> {
    original_event: &'a serde_json::Value,
    pub action: &'a String,
    pub html_url: &'a String,
    pub state: &'a String,
    pub title: &'a String,
}

impl<'a> PullRequestEvent<'a> {
    pub fn new(original_event: &'a serde_json::Value) -> Result<Self, Box<dyn std::error::Error>> {
        let payload = original_event
            .get("payload")
            .map(get_json_object)
            .expect("Ran into a PullRequestEvent without a payload.");
        let payload_action = payload
            .get("action")
            .map(get_json_string)
            .expect("Ran into a PullRequestEvent without a payload.action.");
        let payload_pull_request = payload
            .get("pull_request")
            .map(get_json_object)
            .expect("Ran into a PullRequestEvent without a payload.pull_request.");
        let payload_pull_request_html_url = payload_pull_request
            .get("html_url")
            .map(get_json_string)
            .expect("Ran into a PullRequestEvent without a payload.pull_request.html_url.");
        let payload_pull_request_state = payload_pull_request
            .get("state")
            .map(get_json_string)
            .expect("Ran into a PullRequestEvent without a payload.pull_request.state.");
        let payload_pull_request_title = payload_pull_request
            .get("title")
            .map(get_json_string)
            .expect("Ran into a PullRequestEvent without a payload.pull_request.title.");

        Ok(PullRequestEvent {
            original_event,
            action: payload_action,
            html_url: payload_pull_request_html_url,
            state: payload_pull_request_state,
            title: payload_pull_request_title,
        })
    }

    pub fn is_a(event: &'a serde_json::Value) -> Result<bool, Box<dyn std::error::Error>> {
        let event_type = event.get("type").map(get_json_string);
        match event_type {
            Some(event_type_string) if event_type_string == "PullRequestEvent" => {
                return Ok(true);
            }
            _ => Ok(false),
        }
    }
}
